# Copyright (C) 2019 Hannu Viitala
#
# The source code in this file is released under the MIT license.
# Go to http://opensource.org/licenses/MIT for the full license details.

# Import modules.
import upygame as pygame
import urandom as random
import graphics_data as spritedata
import sprite

# Initialize the display
pygame.display.init()
pygame.display.set_palette_16bit([
    0, 6438, 18917, 10825, 47398, 688, 41764, 17475,
    58225, 13598, 60486, 40179, 42596, 46845, 63245, 65535
]);
screen = pygame.display.set_mode() # full screen

# The GameObject class
class GameObject(sprite.Sprite):
    # Initialize the GameObject. 
    # Parameters:
    # - surfaces: The animation bitmaps.
    # - frameOffsets: The offset for each bitmap.
    def __init__(self, surfaces, frameOffsets):
        sprite.Sprite.__init__(self)  # Call the base class initalizer
        self.frames = surfaces
        self.frameOffsets = frameOffsets
        self.currentFrameNum = 0;
        self.image = self.frames[self.currentFrameNum]  # current image
        self.animDur = 3; # Duration in frames.
        self.animDurCounter = self.animDur;
        self.vx = 0
        self.vy = 0
        self.rect = self.frames[0].get_rect()

    # Set the velocity.
    # Parameters:
    # - vx, vy: The new velocity.
    def setvel(self, vx, vy):
        self.vx = vx
        self.vy = vy

    # Update the position and the animation frame.
    def update(self):

        # Advance frame if the animation is enabled.
        if self.animDur > 0:

            # if animation duration has elapsed, get the next the frame.
            if self.animDurCounter == 0:
                self.currentFrameNum += 1
                if self.currentFrameNum >= len(self.frames):
                    self.currentFrameNum = 0

                # Init the duration counter.
                self.animDurCounter = self.animDur

                # Set the current image and the current position.
                self.image = self.frames[self.currentFrameNum]
                self.rect.x += self.frameOffsets[self.currentFrameNum][0]
                self.rect.y += self.frameOffsets[self.currentFrameNum][1]
            else:
                # Decrement the duration counter.
                self.animDurCounter -= 1

        # Advance the position accordin the speed.
        self.rect.x += self.vx
        self.rect.y += self.vy
        
# Create the sprite groups.
all_sprites = sprite.Group() # Contains all the sprites
all_frogittos = sprite.Group()
all_cars = sprite.Group()

# Create the frogitto sprite.
frogittoGob = GameObject(
    [spritedata.frogittoSurf_f1, spritedata.frogittoSurf_f2, spritedata.frogittoSurf_f3],
    [[0,0],[0,0],[0,0]])
frogittoGob.rect.x = 50
frogittoGob.rect.y = 70  # The bottom of the screen. 
frogittoGob.animDur = 2;

# Add the sprite to the groups.
all_sprites.add(frogittoGob)
all_frogittos.add(frogittoGob)

# Create blue and red car sprites
for i in range(10):

    # Create the car
    carGob = GameObject(
        [spritedata.bluecarSurf_f1, spritedata.bluecarSurf_f2],
        [[0,0],[0,0]])

    # out of screen
    carGob.rect.x = -100
    carGob.rect.y = -100

    # Add the sprite to the groups.
    all_sprites.add(carGob)
    all_cars.add(carGob)

    # Create the car
    carGob = GameObject(
        [spritedata.redcarSurf_f1, spritedata.redcarSurf_f2],
        [[0,0],[0,0]])

    # out of screen
    carGob.rect.x = -100
    carGob.rect.y = -100

    # Add the sprite to the groups.
    all_sprites.add(carGob)
    all_cars.add(carGob)

vx = 0;
vy = 0;
frameNum = 0;
lastY = 0
while True:  # Loop forever.

    # Check collision.
    hit = sprite.spritecollideany(frogittoGob, all_cars)
    if hit != None:
        frogittoGob.rect.x = 50
        frogittoGob.rect.y = 70

    # Read a key.
    eventtype = pygame.event.poll()
    if eventtype != pygame.NOEVENT:
        if eventtype.type == pygame.KEYDOWN:
            if eventtype.key == pygame.K_RIGHT:
                vx = 1
            if eventtype.key == pygame.K_LEFT:
                vx = -1
            if eventtype.key == pygame.K_UP:
                vy = -1
            if eventtype.key == pygame.K_DOWN:
                vy = 1
        if eventtype.type == pygame.KEYUP:
            if eventtype.key == pygame.K_RIGHT:
                vx = 0
            if eventtype.key == pygame.K_LEFT:
                vx = 0
            if eventtype.key == pygame.K_UP:
                vy = 0
            if eventtype.key == pygame.K_DOWN:
                vy = 0

    # Set Frogitto's velocity.
    frogittoGob.setvel(vx, vy);

    # Launch new car after n frames
    if (frameNum % 13) == 0:
        # Get first free car and set the starting position
        for s in all_cars:
            # If the sprite is out of screen, it can be reused.
            if s.rect.x < -16:
                s.rect.x = 110
                y = 10 + random.getrandbits(8) * 60 // 256
                if abs(y - lastY) < 7:
                    if y < lastY:
                        y -= 7
                    else:
                        y += 7
                s.rect.y = y
                lastY = y
                s.setvel(-1,0) # Move to left
                s.currentFrameNum = 0

                break

    # Update and draw all sprites.                
    all_sprites.update()
    all_sprites.draw(screen)

    # Draw the screen buffer to the screen HW.
    pygame.display.flip()

    # Increase the frame number.
    frameNum += 1
    if frameNum > 1000000:
        frameNum = 0;
